#!/bin/bash
# Possible Arguments:
# Boot File
# Root File
# Flash Version Prefix (from which to infer the filenames)
# Repository name (either "local" or <ftp_site>)

# Global Variables, Defaulted
PROGRAM="$0"
VERSION="0.9"
BOOT_FILE="bootfs.jffs2"
ROOT_FILE="rootfs.jffs2"
TEST_MODE=0
INTERACTIVE=1

# System stuff we need:
LOGGER="/usr/bin/logger"
BASENAME="/usr/bin/basename"
CHKFIRMWARE="/home/httpd/jbmconfig/bin/chkfirmware"
DATE="/bin/date"
NANDWRITE="/sbin/nandwrite"
NANDWRITE_STORAGE="/storage/nandwrite"
NANDTEST="/sbin/nandtest"
BTREBOOTER="/bin/btrebooter"
PREPFLASH="/home/httpd/jbmconfig/bin/prepflash.sh"
FLASH_ERASEALL="/sbin/flash_eraseall"
MOUNT="/bin/mount"
SLEEP="/bin/sleep"
REBOOT="/sbin/reboot"
RM="/bin/rm"
CP="/bin/cp"
LS="/bin/ls"
LN="/bin/ln"
DF="/bin/df"
GREP="/bin/grep"
AWK="/bin/awk"

for i in ${LOGGER} ${BASENAME} ${NANDWRITE} ${BTREBOOTER} ${CHKFIRMWARE} ${FLASH_ERASEALL} \
         ${MOUNT} ${SLEEP} ${REBOOT} ${RM} ${CP} ${LS} ${LN} ${DF} ${AWK} ${GREP}
do
    if [ ! -x ${i} ]
    then
        # try our best:
        /usr/bin/logger -t ${PROGRAM} "Required command (${i}) is missing or NOT executable!"
        exit 1
    fi
done

# Adjust some stuff, now that we know we have what we need:
PROGRAM=`${BASENAME} $0`
LOGGER="${LOGGER} -t ${PROGRAM}"

function die {
    (${SLEEP} 30; ${REBOOT}) &
    ${LOGGER} " -- FAILURE. Rebooting in 30 seconds..."
    exit 1
}

# Simple "Usage" function to display help for the user:
function Usage {
cat <<EOF
Usage: ${PROGRAM} <options>

Options:
-h                     Display this help information and exit
-f                     Force non-interactive mode (no prompting)
-b <boot_file_name>    The name of the boot file to use for flashing
-r <root_file_name>    The name of the root file to use for flashing
-t                     Validate procedure, but do NOT flash (Test Mode)
-v                     Display version information and exit
EOF
}

# Process incoming arguments:
while getopts "b:dfhr:s:tv" flag
do
    case ${flag} in
        'b')
            if [ -n "${OPTARG}" -a ${OPTARG:0:1} != "-" ]
            then
                BOOT_FILE="${OPTARG}"
            else
                echo "A boot file name MUST be provided with the '-${flag}' argument... Exiting."
                echo ""
                Usage
           exit 0
            fi
        ;;
        'f')
            INTERACTIVE=0
        ;;
        'h')
            Usage
            exit 0
        ;;
        'r')
            if [ -n "${OPTARG}"  -a ${OPTARG:0:1} != "-" ]
            then
                ROOT_FILE="${OPTARG}"
            else
                echo "A root file name MUST be provided with the '-${flag}' argument... Exiting."
                echo ""
                Usage
           exit 0
            fi
        ;;
        't')
            echo "*********** TEST MODE ***********"
            TEST_MODE=1
        ;;
        'v')
            echo "${PROGRAM} Version ${VERSION}"
            exit 0
        ;;
        *)
            echo ""
            Usage
            exit 1
        ;;
    esac
done


##########################################################################
# Confirm everything with the user before continuing (if interactive)...
##########################################################################
echo "VERSION=${VERSION}"
echo "BOOT_FILE=${BOOT_FILE}"
echo "ROOT_FILE=${ROOT_FILE}"

if [ ${INTERACTIVE} -eq 1 ]
then
    echo ""
    echo -n "Continue [y/n]? "
    read answer

    if [ ${answer} != "y" ]
    then
        exit 0
    fi
fi

${LOGGER} " -- Version ${VERSION} - Started."

#this is for G25 CPU builds, which are not padded
PAD_OPTION=
if [ `ls /sys/devices/platform/at91sam9x5* 2> /dev/null | wc -w` -gt 0 ]; then
   PAD_OPTION="-p"
fi

##########################################################################
# Flash some blinky lights...
##########################################################################
echo 4 > /sys/class/leds/pwr/blink 2> /dev/null
echo 4 > /sys/class/leds/wan/blink 2> /dev/null

##########################################################################
# Make sure we have our image files in place
##########################################################################
# Check to make sure the boot file exists
if [ ! -e "${BOOT_FILE}" ]
then
    echo "${BOOT_FILE} does NOT exist. Abort."
    ${LOGGER} " -- ${BOOT_FILE} does NOT exist. Exiting."
    exit 1
fi

# Check to make sure the root file exists
if [ ! -e "${ROOT_FILE}" ]
then
    echo "${ROOT_FILE} does NOT exist. Abort."
    ${LOGGER} " -- ${ROOT_FILE} does NOT exist. Exiting."
    exit 1
fi

##########################################################################
# Verify JFFS2 image files:
##########################################################################
echo -n "Verifying file integrity..."

CMD="${CHKFIRMWARE} ${BOOT_FILE}"
${CMD}
if [ $? -ne 0 ]
then
    ${LOGGER} " -- '$CMD' failed: $!"
    die
fi

CMD="${CHKFIRMWARE} ${ROOT_FILE}"
${CMD}
if [ $? -ne 0 ]
then
    ${LOGGER} " -- '$CMD' failed: $!"
    die
fi

echo " Complete."


##########################################################################
# Check to see if we are really gonna do this or not:
##########################################################################
if [ $TEST_MODE -eq 1 ]
then
    echo "This is the point of no return..."
    echo ""
    echo "TMP DIR CONTENTS:"
    ${LS} -al /tmp
    echo ""
    echo "STORAGE DIR CONTENTS:"
    ${LS} -al /storage
    echo ""
    echo "FILESYSTEM STATUS:"
    ${DF} -k
    exit 0
fi

##########################################################################
# Tear down more services that may interrupt the process:
##########################################################################
${PREPFLASH}

##########################################################################
# THIS IS THE POINT OF NO RETURN. RESULTS ARE NO LONGER GUARANTEED!!!
##########################################################################
echo "Beginning flash process with:"
echo "BOOT FILE: ${BOOT_FILE}"
echo "ROOT FILE: ${ROOT_FILE}"
${LOGGER} " -- Beginning flash process with:"
${LOGGER} " -- BOOT FILE: ${BOOT_FILE}"
${LOGGER} " -- ROOT FILE: ${ROOT_FILE}"

##########################################################################
# Flash some blinky lights...
##########################################################################
echo 4 > /sys/class/leds/uart/blink 2> /dev/null
echo 2 > /sys/class/leds/reg/blink 2> /dev/null

##########################################################################
# Copy binaries over that we'll need after the ROM is gone:
##########################################################################
echo -n "Staging required files..."

CMD="${CP} ${NANDWRITE} ${NANDWRITE_STORAGE}"
${CMD}
if [ $? -ne 0 ]
then
    ${LOGGER} " -- '${CMD}' failed: $!"
    die
fi

##########################################################################
# Copy the libraries upon which the binaries depend:
##########################################################################
if [ ! -L /lib/libc-2.6.so ]
then
    CMD="${CP} /lib/libc-2.6.so /storage/"
    ${CMD}
    if [ $? -ne 0 ]
    then
        ${LOGGER} " -- '${CMD}' failed: $!"
        die
    fi

    CMD="${LN} -sf /storage/libc-2.6.so /lib/libc.so.6"
    ${CMD}
    if [ $? -ne 0 ]
    then
        ${LOGGER} " -- '${CMD}' failed: $!"
        die
    fi
fi

if [ ! -L /lib/ld-2.6.so ]
then
    CMD="${CP} /lib/ld-2.6.so /storage/"
    ${CMD}
    if [ $? -ne 0 ]
    then
        ${LOGGER} " -- '${CMD}' failed: $!"
        die
    fi

    CMD="${LN} -sf /storage/ld-2.6.so /lib/ld-linux.so.3"
    ${CMD}
    if [ $? -ne 0 ]
    then
        ${LOGGER} " -- '${CMD}' failed: $!"
        die
    fi
fi

echo " Complete."

##########################################################################
# Yank the filesystem away and re-write the flash:
##########################################################################
# Re-mount the filesystem as READ-ONLY:
echo -n "Disabling write access..."

CMD="${MOUNT} -n -o remount,ro /"
${CMD}
if [ $? -ne 0 ]
then
    ${LOGGER} " -- '${CMD}' failed: $!"
    die
fi

echo " Complete."

# Erase the boot partition:
echo "Flashing (will reset in 2 minutes)..."

# Setup a back-door rebooter job:
(${BTREBOOTER} 300) &

echo "Running nandtest on /dev/mtd4"
echo "`${DATE}` - Running nandtest on /dev/mtd4" >> /storage/testnand.log
${NANDTEST} -m /dev/mtd4 | ${AWK} -F "(\r)" '{print $NF}' | ${GREP} -i "bad" >> /storage/testnand.log
${FLASH_ERASEALL} /dev/mtd4

# Replace the boot partition:
${NANDWRITE_STORAGE} ${PAD_OPTION} -a -m /dev/mtd4 "${BOOT_FILE}"

# Erase the root partition:
echo "Running nandtest on /dev/mtd5"
echo "`${DATE}` - Running nandtest on /dev/mtd5" >> /storage/testnand.log
${NANDTEST} -m /dev/mtd5 | ${AWK} -F "(\r)" '{print $NF}' | ${GREP} -i "bad" >> /storage/testnand.log
${FLASH_ERASEALL} /dev/mtd5

# Replace the root partition:
${NANDWRITE_STORAGE} ${PAD_OPTION} -a -m /dev/mtd5 "${ROOT_FILE}"

echo " Complete."

#Reboot
echo "Resetting... "
while [ 1 ]
do
    ${BTREBOOTER}
done

